# Importion of modules
import os
import sys
import pygame
import time

# Own modules
from load_data import *
from code.MagicPotionVessel import *
from code.LiquidMachine import *
from code.Scientist import *
from code.letters import *
from code.LiquidMachineOutputDisplay import *
from code.VesselLiquidInputDisplay import *
from code.Receipe import *
from code.Countdown import *

# Initialize pygame
pygame.init()


class Game( object ):
	"""
	"""

	def __init__( self ):
		"""
		"""

		# Set up the playground.
		self.WIDTH = 800
		self.HEIGHT = 450
		self.COLOUR_DEPTH = 32
		
		# Set the framerate per second.
		self.FRAME_RATE = 60

		# Set up the display.
		self.surface = pygame.display.set_mode( [ self.WIDTH, self.HEIGHT ], 0, self.COLOUR_DEPTH )
		self.surface.fill( ( 10, 10, 10 ) ) # Only temporary.

		# Set up the clock.
		self.clock = pygame.time.Clock()

		


	def run( self ):
		"""
		Runs the game.
		"""

		self.reset()

		#pygame.mixer.music.play()

		while True:

			# Code for quitting.
			eventlist = pygame.event.get()

			for event in eventlist:
				if event.type == pygame.QUIT:
					exit()


			self.surface.fill( ( 0, 0, 0 ) ) # Only temporary.

			# Update everything
			self.liquid_machines.update( self.scientist.chosen_liquid_machine, self.scientist.movements["increase_output"], self.scientist.movements["decrease_output"] )

			self.scientist.update( eventlist )
			self.vessel.update( self.liquid_machines, eventlist )

			if self.receipe.update( eventlist, self.vessel.yellow_liquid, self.vessel.purple_liquid, self.vessel.blue_liquid, self.vessel.green_liquid ):
				self.vessel.yellow_liquid = 0
				self.vessel.purple_liquid = 0
				self.vessel.blue_liquid = 0
				self.vessel.green_liquid = 0

				self.scientist.update_score( self.countdown_timer.time_left )
				self.countdown_timer.reset_timer()
				

			for liquid_machine_display in self.liquid_machines_output_displays:
				if liquid_machine_display.machine_colour == "yellow":
					liquid_machine_display.update( get_liquid_machine_by_colour( self.liquid_machines, "yellow" ).current_liquid_output )
				elif liquid_machine_display.machine_colour == "purple":
					liquid_machine_display.update( get_liquid_machine_by_colour( self.liquid_machines, "purple" ).current_liquid_output )
				elif liquid_machine_display.machine_colour == "blue":
					liquid_machine_display.update( get_liquid_machine_by_colour( self.liquid_machines, "blue" ).current_liquid_output )
				elif liquid_machine_display.machine_colour == "green":
					liquid_machine_display.update( get_liquid_machine_by_colour( self.liquid_machines, "green" ).current_liquid_output )


			for liquid_vessel_display in self.vessel_liquid_input_number_displays:
				if liquid_vessel_display.liquid_colour == "yellow":
					liquid_vessel_display.update( self.vessel.yellow_liquid )
				elif liquid_vessel_display.liquid_colour == "purple":
					liquid_vessel_display.update( self.vessel.purple_liquid )
				elif liquid_vessel_display.liquid_colour == "blue":
					liquid_vessel_display.update( self.vessel.blue_liquid )
				elif liquid_vessel_display.liquid_colour == "green":
					liquid_vessel_display.update( self.vessel.green_liquid )


			# Update the countdown timer.
			if self.countdown_timer.update():
				self.receipe.generate_receipe()
				self.receipe.image.fill( ( 0, 0, 0 ) )
				self.receipe.blit_receipe()
				self.vessel.yellow_liquid = 0
				self.vessel.purple_liquid = 0
				self.vessel.blue_liquid = 0
				self.vessel.green_liquid = 0
			else:
				pass


			if self.total_time_left.update():
				# Display the score.
				self.surface.fill( ( 0, 0, 0 ) )
				score = Word( "YOUR SCORE: " + str( self.scientist.score ), ( 150, 200 ), ( ( 255, 255, 255 ) ), 7 )
				message = Word( "WILL RETURN TO MAIN MENU IN 5 SECONDS", ( 150, 300 ), ( ( 255, 255, 255 ) ), 2 )
				self.surface.blit( score.image, score.rect )
				self.surface.blit( message.image, message.rect )
				pygame.display.flip()
				time.sleep( 5 )
				#pygame.mixer.music.stop()
				break
			
			# Draw everything.
			self.liquid_machines.draw( self.surface )
			for monitor in self.liquid_output_monitors:
				self.surface.blit( monitor[0], monitor[1] )

			for monitor in self.liquid_input_monitors:
				self.surface.blit( monitor[0], monitor[1] )

								
			self.liquid_machines_output_displays.draw( self.surface )
			self.vessel_liquid_input_number_displays.draw( self.surface )
			self.surface.blit( self.receipe.image, self.receipe.rect )
			self.surface.blit( self.countdown_timer.image, self.countdown_timer.rect )
			self.surface.blit( self.total_time_left.image, self.total_time_left.rect )
			self.surface.blit( self.scientist.image, self.scientist.rect )
			
			# Update the display.
			pygame.display.flip()
			self.clock.tick( self.FRAME_RATE )

	def display_menu( self ):
		"""
		Creates the menu of the game.
		"""

		self.menu = Menu()

		while True:

			eventlist = pygame.event.get()

			# Update
			#self.menu.change_heading_colour()
			self.menu.update()

			for btn in self.menu.buttons:
				if btn in self.menu.active_buttons and btn.name == "start_game_btn" and pygame.mouse.get_pressed()[0] == True:
					self.run()
				elif btn in self.menu.active_buttons and btn.name == "help_game_btn" and pygame.mouse.get_pressed()[0] == True:
					pass
				elif btn in self.menu.active_buttons and btn.name == "exit_game_btn" and pygame.mouse.get_pressed()[0] == True:
					exit()
				else:
					pass
			
			# Draw
			self.surface.blit( self.menu.image, self.menu.rect )

			self.clock.tick( self.FRAME_RATE )

			pygame.display.flip()


	def reset( self ):
		"""
		"""

		self.create_objects()


	def create_objects( self ):
		"""
		This method creates the objects for the game.
		"""
		
		# Create the liquid machines objects.
		self.create_liquid_machine_objects()

		# Create the player ( the scientist ).
		self.scientist = Scientist( len( self.liquid_machines ) )

		# Create the magic potion vessel.
		self.vessel = MagicPotionVessel( magic_potion_vessel_image, [ 0, 400 ] );

		# Create the liquid machine liquid output displays.
		self.create_liquid_machine_output_displays()

		# Create the liquid machine liquid output monitors.
		self.create_liquid_output_monitors()

		# Create the magic vessel liquid statisitcs displays.
		self.create_vessel_liquid_input_number_displays()
		self.create_liquid_input_monitors()

		# Generate the receipe
		self.receipe = Receipe()

		# Create the countdown timer.
		self.countdown_timer = Countdown( ( 190, 10 ), 120, "TIME LEFT ", 4)

		# Create the total time left timer.
		self.total_time_left = Countdown( ( 520, 10 ), 300, "TOTAL TIME LEFT ", 2 )
		

	def create_liquid_machine_objects( self ):
		"""
		This method creates the 4 liquid machine objects.
		"""

		# The liquid machines are stored in an extra sprite group.
		self.liquid_machines = pygame.sprite.Group()

		# Create the objects now.
		self.liquid_machines.add( LiquidMachine( liquid_machine_yellow_image, liquid_machine_yellow_chosen_image, [ 195, 280 ], "yellow", 0 ) )
		self.liquid_machines.add( LiquidMachine( liquid_machine_purple_image, liquid_machine_purple_chosen_image,[ 320, 280 ], "purple", 1 ) )
		self.liquid_machines.add( LiquidMachine( liquid_machine_blue_image, liquid_machine_blue_chosen_image, [ 445, 280 ], "blue", 2 ) )		
		self.liquid_machines.add( LiquidMachine( liquid_machine_grey_image, liquid_machine_grey_chosen_image, [ 570, 280 ], "green", 3 ) )	




	def create_liquid_machine_output_displays( self ):
		"""
		"""

		self.liquid_machines_output_displays = pygame.sprite.Group()

		# And now create the objects.
		self.liquid_machines_output_displays.add( LiquidMachineOutputDisplay( "0", [ 725, 63 ], ( 250, 240, 0 ), 3, "yellow" ) )
		self.liquid_machines_output_displays.add( LiquidMachineOutputDisplay( "0", [ 725, 163 ], ( 180, 0, 100 ), 3, "purple" ) )
		self.liquid_machines_output_displays.add( LiquidMachineOutputDisplay( "0", [ 725, 263 ], ( 0, 50, 200 ), 3, "blue" ) )
		self.liquid_machines_output_displays.add( LiquidMachineOutputDisplay( "0", [ 725, 363 ], ( 20, 200, 30 ), 3, "green" ) )				


	def create_liquid_output_monitors( self ):
		"""
		"""
		
		self.liquid_output_monitors = []
		self.liquid_output_monitors.append( [ display_liquid_output_yellow.convert_alpha(), [ 700, 0, 74, 48 ] ] )
		self.liquid_output_monitors.append( [ display_liquid_output_purple.convert_alpha(), [ 700, 100, 74, 48 ] ] )
		self.liquid_output_monitors.append( [ display_liquid_output_blue.convert_alpha(), [ 700, 200, 74, 48 ] ] )
		self.liquid_output_monitors.append( [ display_liquid_output_green.convert_alpha(), [ 700, 300, 74, 48 ] ] )						




	def create_vessel_liquid_input_number_displays( self ):
		"""
		"""

		self.vessel_liquid_input_number_displays = pygame.sprite.Group()

		# And now create the objects.
		self.vessel_liquid_input_number_displays.add( VesselLiquidInputDisplay( "0", [ 45, 63 ], ( 250, 240, 0 ), 3, "yellow" ) )
		self.vessel_liquid_input_number_displays.add( VesselLiquidInputDisplay( "0", [ 45, 163 ], ( 180, 0, 100 ), 3, "purple" ) )
		self.vessel_liquid_input_number_displays.add( VesselLiquidInputDisplay( "0", [ 45, 263 ], ( 0, 50, 200 ), 3, "blue" ) )
		self.vessel_liquid_input_number_displays.add( VesselLiquidInputDisplay( "0", [ 45, 363 ], ( 20, 200, 30 ), 3, "green" ) )


	def create_liquid_input_monitors( self ):
		"""
		"""

		self.liquid_input_monitors = []
		self.liquid_input_monitors.append( [  display_liquid_input_yellow.convert_alpha(), [ -50, 0, 250, 48 ] ] )
		self.liquid_input_monitors.append( [  display_liquid_input_purple.convert_alpha(), [ -50, 100, 250, 48 ] ] )
		self.liquid_input_monitors.append( [  display_liquid_input_blue.convert_alpha(), [ -50, 200, 250, 48 ] ] )
		self.liquid_input_monitors.append( [  display_liquid_input_green.convert_alpha(), [ -50, 300, 250, 48 ] ] )						





def get_liquid_machine_by_colour( liquid_machines, colour ):
	"""
	This function returns the object of the liquid machine with the colour specified.
	"""

	for machine in liquid_machines:
		if machine.name_colour == colour:
			return machine


def get_liquid_machine_output_display_by_colour( liquid_machine_output_displays, colour ):
	"""
	This function sreturns the object of the liquid machine output display of which the colour has been specified.
	"""

	for display in liquid_machine_output_displays:
		if display.machine_colour == colour:
			return display



class Menu( object ):
	"""
	Represents the menu.
	"""

	def __init__( self ):
		"""
		Initialize the object.
		"""

		self.image = pygame.Surface( ( 800, 400 ) )
		self.rect = self.image.get_rect()
		self.image.fill( ( 0, 0, 0 ) )

		self.change_heading_every = 60 # frames
		self.change_heading_counter = 0

		self.btn_active_colour = ( 200, 200, 200 )
		self.btn_passive_colour = ( 100, 100, 100 )

		self.heading = Word( "SCIENTIFIC LIQUIDS", ( 50, 50 ), (( 255, 255, 255 )), 3 )

		self.image.blit( self.heading.image, self.heading.rect )

		self.create_buttons()


	def update( self ):
		"""
		"""
		self.buttons.draw( self.image )
		self.mouse_over_button()


	def change_heading_colour( self ):
		"""
		Creates a new Word object and changes the colour of the heading.
		"""

		if self.change_heading_counter >= self.change_heading_every:
			self.change_heading_counter = 0
			self.heading = Word( "LIQUID SCIENCE", ( 50, 50 ), ( random.randint( 0, 255 ), random.randint( 0, 255 ), random.randint( 0, 255 ) ), 3 )
			self.image.blit( self.heading.image, self.heading.rect )
		else:
			self.change_heading_counter += 1


	def create_buttons( self ):
		"""
		This functions creates the buttons for the menu.
		"""

		self.passive_buttons = []
		self.active_buttons = []

		self.buttons = pygame.sprite.Group()

		self.btn_start_game_passive = Word( "START GAME", ( 50, 100 ), self.btn_passive_colour, 2 )
		self.btn_start_game_passive.name = "start_game_btn"
		self.btn_start_game_active = Word( "START GAME", ( 50, 100 ), self.btn_active_colour, 2 )
		self.btn_start_game_active.name = "start_game_btn"
		self.btn_game_help_passive = Word( "HELP", ( 50, 135 ), self.btn_passive_colour, 2 )
		self.btn_game_help_passive.name = "help_game_btn"
		self.btn_game_help_active = Word( "HELP", ( 50, 135 ), self.btn_active_colour, 2 )
		self.btn_game_help_active.name = "help_game_btn"		
		self.btn_game_exit_passive = Word( "EXIT GAME", ( 50, 170 ), self.btn_passive_colour, 2 )
		self.btn_game_exit_passive.name = "exit_game_btn"		
		self.btn_game_exit_active = Word( "EXIT GAME", ( 50, 170 ), self.btn_active_colour, 2 )
		self.btn_game_exit_active.name = "exit_game_btn"

		self.buttons.add( self.btn_start_game_passive )
		self.buttons.add( self.btn_game_help_passive )
		self.buttons.add( self.btn_game_exit_passive )

		self.active_buttons.append( self.btn_start_game_active )
		self.active_buttons.append(self.btn_game_help_active )
		self.active_buttons.append(self.btn_game_exit_active )

		 
		self.passive_buttons.append( self.btn_start_game_passive )
		self.passive_buttons.append( self.btn_game_help_passive )
		self.passive_buttons.append( self.btn_game_exit_passive )


	def get_button( self, btn_list, btn_name ):
		"""
		Returns the button to add to self.buttons.
		"""
		for btn in btn_list:
			if btn.name == btn_name:
				return btn

	def mouse_over_button( self ):
		"""
		This function checks and returns either True or False, depending on if the mouse is over a button.
		"""

		mouse_pos = pygame.mouse.get_pos()

		for button in self.passive_buttons:
			if button.rect.collidepoint( mouse_pos ):
				btn_to_add = self.get_button( self.active_buttons, button.name )
				if not btn_to_add in self.passive_buttons:
					self.buttons.add( btn_to_add )
					self.buttons.remove( button )
				else:
					pass
			else:
				pass


		for button in self.active_buttons:
			if not button.rect.collidepoint( mouse_pos ):
				btn_to_add = self.get_button( self.passive_buttons, button.name )
				if not btn_to_add in self.active_buttons:
					self.buttons.add( btn_to_add )
					self.buttons.remove( button )
				else:
					pass
			else:
				pass
					
				

		
		
		
			
# Run the game
game = Game()
game.display_menu()
#game.run()
